﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Markup;
using System.Diagnostics;
using System.Windows.Browser;
using SilverlightMouseSupport;

namespace MouseWheelEnabledControls
{
    /// <summary>
    /// MouseSupport is a decorator which is used to add mouse events to Silverlight controls.
    /// It has been tested on Microsoft Internet Explorer 7 and Firefox versions 2 and 3.
    /// Known issues/restrictions:
    ///     - When switching to full screen Silverlight, the mouse wheel events are no longer being send to the control.
    ///     - The MouseWheel event is NOT a routed event. To catch the MouseWheel event on a parent decorate the parent with its own MouseSupport decorator.
    ///     - The MouseSupport decorator has several properties like Background, BorderBrush etc. that should not be used the control should remain UI-less.
    ///     - The MouseSupport decorator has a Template property but it can NOT be set.
    /// </summary>
    public class ComboBox: System.Windows.Controls.ComboBox
    {
        private string _originalTemplate;

        private MouseSupport _mouseSupport;


        /// <summary>
        /// Builds the visual tree for the MouseSupport decorator control when a new template is applied.
        /// </summary>
        public override void OnApplyTemplate()
        {

            ControlTemplate t = this.Template;
            // Get the root visual of the template
            ChildElement = GetTemplateChild(ChildElementName) as ContentPresenter;
            ChildElement.MouseEnter += new MouseEventHandler(ChildElement_MouseEnter);
            ChildElement.MouseLeave += new MouseEventHandler(ChildElement_MouseLeave);
            Debug.Assert(ChildElement != null, "The required template part ChildElement was not found!");
        }

        override 
        public ComboBox()
        {
            ControlTemplate originalTemplate = Template;
            Template = DecorateTemplate(originalTemplate);
            ApplyTemplate();
            AttachBrowserMouseWheelEvent();
        }

        private ControlTemplate DecorateTemplate(ControlTemplate originalTemplate)
        {
            ControlTemplate decoratedTemplate;
            string template = XamlWriter
            return decoratedTemplate;
        }


        #region MouseWheel Functionalty

        /// <summary>
        /// The MouseWheel event is raised when the mouse wheel is being rolled and the mouse is over the child control of this decorator.
        /// </summary>
        public event EventHandler<MouseWheelEventArgs> MouseWheel;

        private bool _isMouseOverChild = false;

        /// <summary>
        /// AttachBrowserMouseWheelEvent attaches the decorator to the browser events.
        /// </summary>
        private void AttachBrowserMouseWheelEvent()
        {
            if (HtmlPage.IsEnabled)
            {
                if (HtmlPage.BrowserInformation.UserAgent.Contains("Firefox"))
                {
                    HtmlPage.Plugin.AttachEvent("DOMMouseScroll", this.ProcessOnMouseWheel);
                }
                if (HtmlPage.BrowserInformation.UserAgent.Contains("MSIE") ||
                    HtmlPage.BrowserInformation.UserAgent.Contains("Opera") ||
                    HtmlPage.BrowserInformation.UserAgent.Contains("Safari"))
                {
                    HtmlPage.Plugin.AttachEvent("onmousewheel", this.ProcessOnMouseWheel);
                }
            }
        }

        /// <summary>
        /// ProcessOnMouseWheel processes the browser's event mouse wheel data and raises the event when the mouse is over the decorated child.
        /// When the MouseWheel event returns the Handled property of the MouseWheelEventArgs is inspected.
        /// If Handled is true the mouse wheel event will not be propagated to the browser so the browser will not scroll or zoom.
        /// </summary>
        public void ProcessOnMouseWheel(object sender, HtmlEventArgs e)
        {
            if (_isMouseOverChild)
            {
                // see: http://blog.bodurov.com/Cross-browser-mouse-wheel-control
                // for browser specific information on the event data 
                double delta = 0;

                ScriptObject eventObj = e.EventObject;

                if (eventObj.GetProperty("wheelDelta") != null)
                {
                    delta = ((double)eventObj.GetProperty("wheelDelta")) / 120;


                    if (HtmlPage.Window.GetProperty("opera") != null)
                        delta = -delta;
                }
                else if (eventObj.GetProperty("detail") != null)
                {
                    delta = -((double)eventObj.GetProperty("detail")) / 3;

                    if (HtmlPage.BrowserInformation.UserAgent.Contains("Macintosh"))
                        delta = delta * 3;
                }

                if (delta != 0)
                {
                    bool handled = OnMouseWheel(delta);
                    if (handled)
                    {
                        e.PreventDefault();
                    }
                }
            }
        }

        /// <summary>
        /// Raises the MouseWheel event.
        /// </summary>
        /// <param name="delta">Contains the amount the mouse wheel has been scrolled.</param>
        /// <returns>true when the mouse wheel event has been handled and should not be propagated to the browser.</returns>
        protected bool OnMouseWheel(double delta)
        {
            EventHandler<MouseWheelEventArgs> m = MouseWheel;
            if (m != null)
            {
                MouseWheelEventArgs e = new MouseWheelEventArgs(delta);
                m(this.Child, e);
                return e.Handled;
            }
            else
            {
                return false;
            }
        }

        /// <summary>
        /// Registers the mouse leaving the decorated child control.
        /// </summary>
        private void ChildElement_MouseLeave(object sender, MouseEventArgs e)
        {
            _isMouseOverChild = false;
        }

        /// <summary>
        /// Registers the mouse entering the decorated child control.
        /// </summary>
        private void ChildElement_MouseEnter(object sender, MouseEventArgs e)
        {
            _isMouseOverChild = true;
        }

        #endregion
    }
}
